# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os

from mirage          import fileable
from mirage.flow     import Workflow
from mirage.command  import command, log, raise_error_message
from mirage.miragefile import source
from mirage.configure.miragefile import create_additional


class ConfigureWorkFlow(Workflow):

    def constructor(self):
        self._configure_target = self._option

    def main(self):
        if self._configure_target == "addon":
            self._configure_addition()
        elif self._configure_target == "secret":
            self._configure_secret()
        else:
            self._configure()

    def _configure(self):

        if fileable.exists("Miragefile"):
            if log("Miragefile is exists. Are you sure to overwrite Miragefile?", withConfirm = True):
                os.remove("Miragefile")
            else:
                log("Miragefile is already exists!", withError = True)
                raise FileExistsError
                return


        app_name     = log("App name", withInput = True)
        version      = log("App version", withInput = True)
        author       = log("Author name", withInput = True)
        email        = log("Email", withInput = True)
        git_url      = log("Git URL", withInput = True)
        license_name = log("License", withInput = True)
        description  = log("Description", withInput = True)
        copyrightor  = log("Copyrightor", withInput = True, default = author)


        with open("Miragefile", "w") as f:
            f.write(source.create(app_name, version, author, email, git_url, license_name, description, copyrightor))

    def _configure_addition(self):
        if fileable.exists("Miragefile.addon"):
            if log("Miragefile (Additional) is exists. Are you sure to overwrite?", withConfirm = True):
                os.remove("Miragefile.addon")
            else:
                log("Miragefile is already exists!", withError = True)
                raise FileExistsError
                return

        option_string = log("Additional option string", withInput = True)

        with open("Miragefile.addon", "w") as f:
            f.write(create_additional(option_string))


    def _configure_secret(self):
        with open("Miragefile.secret", "w") as f:
            f.write("")



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

###########  WARNING  #####################################################
# This module will be discarded. Please replace new API mirage.miragefile #
###########################################################################

import os
import enum
import json
from mirage import system as sys
from mirage.command import log, raise_error_message


class MiragefileDataCategory(enum.Enum):
    project_name = 0
    project_version = 1
    project_author = 2
    project_git = 3
    project_license = 4
    project_description = 5
    django_path = 6
    django_module = 7
    django_package_manager = 8
    django_db_backend = 9
    front_path = 10
    front_package = 11
    front_builder = 12
    workspace_path = 13
    copyright_start_year = 14
    copyright_copyrigtors = 15


def get_project(item):
    
    data = load_miragefile()

    if item == MiragefileDataCategory.project_name:
        return data["project"]["name"]
    elif item == MiragefileDataCategory.project_version:
        return data["project"]["version"]
    elif item == MiragefileDataCategory.project_author:
        return data["project"]["author"]
    elif item == MiragefileDataCategory.project_git:
        return data["project"]["git"]
    elif item == MiragefileDataCategory.project_license:
        return data["project"]["license"]
    elif item == MiragefileDataCategory.project_description:
        return data["project"]["description"]
    else:
        log("The config information named " + item + " does not exist!", withError = True) 
        return load_failed()


def get_django(item):

    data = load_miragefile()

    if item == MiragefileDataCategory.django_path:
        return data["project"]["django"]["path"]
    elif item == MiragefileDataCategory.django_module:
        return data["project"]["django"]["module"]
    elif item == MiragefileDataCategory.django_package_manager:
        return data["project"]["django"]["package"]
    elif item == MiragefileDataCategory.django_db_backend:
        return data["project"]["django"]["database"]
    else:
        log("The config information named " + item + " does not exist!", withError = True) 
        return load_failed()


def get_copyright(item):

    data = load_miragefile()

    if item == MiragefileDataCategory.copyright_start_year:
        return data["project"]["copyright"]["start_year"]
    elif item == MiragefileDataCategory.copyright_copyrigtors:
        return data["project"]["copyright"]["copyrightors"]
    else:
        log("The config information named " + item + " does not exist!", withError = True) 
        return load_failed()


def get_private_profile(item):

    data = load_miragefile_secret()

    if item == "name":
        return data["private_profile"]["name"]
    elif item == "license":
        return data["private_license"]["url"]
    else:
        log("The config information named " + item + " does not exist!", withError = True) 
        return load_failed()



def get_reserved_addon_config(item):

    data = load_miragefile_addon()

    if item == "iyashi":
        try:
            return data["additional_options"]["iyashi"]
        except:
            return False
    else:
        log("The config information named " + item + " does not exist!", withError = True) 
        return load_failed()


def load_miragefile():
    return _load_json("Miragefile")


def load_miragefile_addon():
    return _load_json("Miragefile.addon")


def load_miragefile_secret():
    return _load_json("Miragefile.secret")


def _load_json(filename):

    if not os.path.exists(filename):
        return filename + " does not exist!"

    with open(filename, "r") as jsonfile:
        try: 
            return json.load(jsonfile)
        except:
            log("Failed to load Miragefile!", withError = True, errorDetail = raise_error_message(_load_json))
            return load_failed()

def load_failed():
    return "Invalid Miragefile"



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import platform
import sys
import textwrap

try: # pip >= 10
    from pip._internal.utils.misc import get_installed_distributions
except ImportError:  # pip < 10
    from pip import get_installed_distributions

from mirage.command import log


def src(name, desc = "No description provided."):

    return textwrap.dedent(
'''
# {NAME}

{DESCRIPTION}

# Info
Information of development environment.

## Environment
OS: {OS}

## Versions
Django Version: `{DJANGO_VER}`

Python Version: `{PYTHON_VER}`

## Installed pip packages
{INSTALLED_PIP_PACKAGES}

''').format(
    NAME = name,
    OS = get_os_name(),
    DJANGO_VER = get_django_version(),
    PYTHON_VER = get_python_version(),
    INSTALLED_PIP_PACKAGES = get_pip_list(),
    DESCRIPTION = desc,
).strip()


def get_django_version():
    try:
        import django
        version = django.VERSION
        return str(version[0]) + "." + str(version[1]) + "." + str(version[2])
    except:
        log("Failed to import Django!", withError = True)
        return "FAILED TO IMPORT DJANGO!"


def get_python_version():
    version = sys.version_info
    return str(version[0]) + "." + str(version[1]) + "." + str(version[2])


def get_os_name():
    os = platform.system()

    if os == "Darwin":
        return "macOS"
    elif os == "Windows":
        return os
    else:
        return os


def get_pip_list():
    string = ""

    ignore_packages = ["setuptools", "pip", "python", "mirage"]

    packages = get_installed_distributions(local_only = True, skip = ignore_packages)

    for package in packages:
        string += "+ " + package.project_name + " " + package.version + "  \n"

    return string



# -*- coding: utf-8 -*-
"""
Copyright 2017 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import shutil
import tarfile
import os
from mirage.command import log, raise_error_message
from mirage.backup import backup


def _check(name):
    if os.path.isdir(name):
        return
    else:
        raise FileNotFoundError


def _backup(name):
    try:
        _check(name)
    except:
        return


    backupper = backup.DjangoBackupAppWorkFlow([
            "b",
            None,
            "app",
            None,
            name
    ])

    backupper.main()

   
def _destroy(name):
    shutil.rmtree(name)



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys, os, imp
from mirage.core import Void
from mirage import system as mys
from mirage.proj import MirageEnvironment, MirageEnvironmentLevel


class ConfigScript():

    def __init__(self):

        # Configs
        self._enable_log: bool
        self._allow_interactive: bool

        # Setting values
        self._custom_clis = []
        self._custom_scripts = []

        # Auto exe funcs
        self.initialize()
        self.main()

    def __del__(self):
        self.deinitialize()


    def initialize(self) -> Void:
        ...


    def deinitialize(self) -> Void:
        ...

    def main(self) -> int:
        return 0


    @classmethod
    def register_prescript(self, script_path: str, target_func: str, runtime = "python") -> int:
        mys.log("Sorry, Mirage Prescript is now under construction. ; (", withError = True)
        return 0


    @classmethod
    def register_postscript(self, script_path: str, target_func: str, runtime = "python") -> int:
        mys.log("Sorry, Mirage Prescript is now under construction. ; (", withError = True)
        return 0


    def register_custom_command(self, cli: str, option: str, script_path: str) -> int:
        self._custom_clis.append((cli, option, script_path))

    def register_custom_command_with_runtime(self, cli: str, script_path: str, runtime: str) -> int:
        self._custom_scripts.append((cli, script_path, runtime))


    def execute(self, cli: str, option: str) -> int:

        for normal_cli in self._custom_clis:
            pass

        for runtime_cli in self._custom_scripts:
            pass


    @staticmethod
    def assertBool() -> bool:
        return True



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

def src():
    return '''
#
# .gitignore for Mirage project
#
#  Version 2018/3/25
#
# Based on github useful .gitignore collections
# https://github.com/github/gitignore
#

#########################################################
## Python Ignore                                       ##
#########################################################

# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
.hypothesis/
.pytest_cache/

# Translations
*.mo
*.pot

# Django stuff:
*.log
.static_storage/
.media/
local_settings.py

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
target/

# Jupyter Notebook
.ipynb_checkpoints

# pyenv
.python-version

# celery beat schedule file
celerybeat-schedule

# SageMath parsed files
*.sage.py

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/


#########################################################
## Node.js Ignore                                      ##
#########################################################

# Logs
logs
*.log
npm-debug.log*
yarn-debug.log*
yarn-error.log*

# Runtime data
pids
*.pid
*.seed
*.pid.lock

# Directory for instrumented libs generated by jscoverage/JSCover
lib-cov

# Coverage directory used by tools like istanbul
coverage

# nyc test coverage
.nyc_output

# Grunt intermediate storage (http://gruntjs.com/creating-plugins#storing-task-files)
.grunt

# Bower dependency directory (https://bower.io/)
bower_components

# node-waf configuration
.lock-wscript

# Compiled binary addons (https://nodejs.org/api/addons.html)
build/Release

# Dependency directories
node_modules/
jspm_packages/

# Typescript v1 declaration files
typings/

# Optional npm cache directory
.npm

# Optional eslint cache
.eslintcache

# Optional REPL history
.node_repl_history

# Output of 'npm pack'
*.tgz

# Yarn Integrity file
.yarn-integrity

# dotenv environment variables file
.env

# next.js build output
.next


#########################################################
## Django Ignore                                       ##
#########################################################

*.pyc
*.sqlite3


#########################################################
## Mirage Ignore                                       ##
#########################################################

.mirage/
Miragefile.addon
Miragefile.secret


#########################################################
## Front-end Ignore                                    ##
#########################################################

shell/dist/
shell/build/

#########################################################
## Project Ignore                                      ##
#########################################################

tmp/

# Describe project specified ignore files or directories here.

'''



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os

try: # pip >= 10
    from pip._internal.utils.misc import get_installed_distributions
except ImportError:  # pip < 10
    from pip import get_installed_distributions

from mirage.flow import Workflow
from mirage.command import command
from mirage import system as mys


class DjangoPackageWorkFlow(Workflow):

    def constructor(self):
        
        self._action = self._option
        self._packages = self._values

    def main(self):
        mys.log("Mirage package manager...")

    def _init(self):
        ignore_packages = ["setuptools", "pip", "python"]
        already_pip = get_installed_distributions(local_only = True, skip = ignore_packages)


    def _install(self, package_name):
        mys.log("Installing package " + package_name + " ...")

    def _uninstall(self, package_name):
        mys.log("Uninstalling package " + package_name + " ...")


    def _integrate(self, pipfile):
        pass



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import readline
from mirage.core import Void
from mirage.flow import Workflow
from mirage.generate.urlpy import create_urlpy_script as url_script
from mirage import system as mys
from mirage import proj



class DjangoAppMakeWorkFlow(Workflow):

    def constructor(self) -> Void:
        self._must_creat_apps = self._values


    def main(self) -> bool:
        try:
            self._check_all()
        except:
            mys.log("Environmental compatability is invalid.", withError = True)
            return


        for app in self._must_creat_apps:
            mys.log("Creating a app " + app + ".")
            self._create_app(app)
            self._create_url(app)
            self._install_app(app)

        return True


    def _create_app(self, name):
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):
            mys.command("python manage.py startapp " + name)


    def _create_url(self, name):
        with proj.InDir("./" + name):
            with open("urls.py", "a") as newscript:
                newscript.write(url_script(name))


    """
    Install created app to Django project
    """
    def _install_app(self, name):

        try:
            master_app = self.__detect_master_app()
        except:
            mys.log("Failed to detect master app.", withError = True)


        mys.log("Installing app...")

        with proj.InDir(master_app):
            if os.path.isdir("environment"):
                with proj.InDir("./environment"):
                    self.__insert_app_path(name, "common.py")

            elif os.path.isdir("settings"):
                with proj.InDir("./settings"):
                    self.__insert_app_path(name, "common.py")

            else:
                if os.path.isfile("settings.py"):
                    self.__insert_app_path(name, "settings.py")
                else:
                    mys.log("Failed to install Django app due to missing configuration file.", withError = True)


    """
    Check compatibility of Django.
    """
    def _check_all(self):

        reserved_names = (
            "test",
            "os"
        )

        for app in self._must_creat_apps:
            for name in reserved_names:
                if app == name:
                    mys.log("The app named " + app + " is reserved by Django!", withError = True)
                    raise ValueError

        for app in self._must_creat_apps:
            if os.path.isdir(app):
                mys.log("The app named " + app + " is already exists.", withError = True)
                raise FileExistsError


    def __detect_master_app(self):
        
        dirs = os.listdir(os.getcwd())

        current = os.getcwd()

        for app in dirs:

            try:
                os.chdir("./" + app)
                if os.path.isfile("settings.py"):
                    os.chdir(current)
                    return app
                else:
                    os.chdir(current)
            except: 
                pass
       
        return None


    def __insert_app_path(self, app_name, setting_file):
        lines = []
        insert_line = 0

        try:
            with open(setting_file, "r") as setting:
                try:
                    lines = setting.readlines()
                except:
                    mys.log("Failed to load configuration file lines.", withError = True)
        except:
            pass

    
        for i in range(len(lines)):
            if "INSTALLED_APPS = [" in lines[i]:
                insert_line = i
            elif "INSTALLED_APPS = (" in lines[i]:
                insert_line = i
    
        with open(setting_file, "w") as setting:
            app_config = app_name[0].upper() + app_name[1:] + "Config"
            lines.insert(insert_line + 1, "    \'" + app_name + ".apps." + app_config + "\',\n")
            setting.writelines(lines)



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import enum
from mirage import system as sys
from mirage.flow import Workflow
from .model_template import create_model_class


class DjangoModelMakeWorkflow(Workflow):

    def constructor(self):
        self._model_name = self._values[0]
        self._values.pop(0)
        self._data = self._values

    def main(self):
        # ModelClass name:string+maxlen=40 no:integer+as_primary=True

        model_contents = ""

        for data in self._data:
            model_contents += "{0}\n".format(self._parse_data(data))

        # Write model file
        sys.log("Writing model...")

        try:
            with open("models.py", "a") as writing:
                writing.write(create_model_class(self._model_name, model_contents))
        except:
            sys.log("Failed to create / overwrite models.py!", withError = True,
                                            errorDetail = sys.raise_error_message(self.main))


    def _parse_data(self, data_string):

        # data_name:data_type+option=value,option=value
        # ex. name:string+maxlen=100,as_primary=True

        data_name = data_string.split(":")[0]
        data_type = self._parse_data_type(data_string)
        data_options = self._parse_option(data_string)

        return self._make_col(data_name, data_type, data_options)


    def _parse_data_type(self, data_string):
        if "+" in data_string:
            return data_string.split(":")[1].split("+")[0]
        else:
            return data_string.split(":")[1]


    def _parse_option(self, data_string):
        if "+" in data_string:
            options = data_string.split("+")[1].split(",")

            options_parsed = []

            for option in options:
                option_name = option.split("=")[0]
                option_value = option.split("=")[1]

                options_parsed.append((option_name, option_value))

            return options_parsed
        else:
            return None


    def _make_col(self, name, type, ops):
        string = "  {0} = {1}({2})".format(name, self._make_filed(type), self._make_option(ops, type))

        return string


    def _make_filed(self, type):

        if type == "string":
            return "models.CharField"
        elif type == "auto":
            return "models.AutoField"
        elif type == "auto64":
            return "models.BigAutoField"
        elif type == "int64":
            return "models.BigIntegerField"
        elif type == "binary":
            return "models.BinaryField"
        elif type == "bool":
            return "models.BooleanField"
        elif type == "char":
            return "models.CharField"
        elif type == "date":
            return "models.DateField"
        elif type == "datetime":
            return "models.DateTimeField"
        elif type == "decimal":
            return "models.DecimalField"
        elif type == "duration" or type == "interval":
            return "models.DurationField"
        elif type == "email":
            return "models.EmailField"
        elif type == "file":
            return "models.FileField"
        elif type == "path":
            return "models.FilePathField"
        elif type == "float":
            return "models.FloatField"
        elif type == "image":
            return "models.ImageField"
        elif type == "int" or type == "integer":
            return "models.IntegerField"
        elif type == "ip":
            return "models.GenericIPAddressField"
        elif type == "nullbool":
            return "models.NullBooleanField"
        elif type == "pint":
            return "models.PositiveIntegerField"
        elif type == "slug":
            return "models.SlugField"
        elif type == "text":
            return "models.TextField"
        elif type == "time":
            return "models.TimeField"
        elif type == "url":
            return "models.URLField"
        elif type == "uuid":
            return "models.UUIDField"


    def _make_option(self, ops, data_type):

        if ops == None:
            if data_type == "string": return "max_length = 255"
            else: return ""

        opstring = ""

        # Max len check
        text_is_maxlen = False
        for op in ops:
            if "maxlen" in op[0]: text_is_maxlen = True

        if data_type == "string" and text_is_maxlen == False:
            opstring += "max_length = 255, "

        # Generate filed options
        for op in ops:
            if op[0] == "maxlen":
                opstring += "max_length = {0}".format(op[1])
            elif op[0] == "primary":
                opstring += "primary_key = {0}".format(op[1])
            elif op[0] == "default":
                opstring += "default = {0}".format(op[1])
            elif op[0] == "blank":
                opstring += "blank = {0}".format(op[1])
            elif op[0] == "null":
                opstring += "null = {0}".format(op[1])
            elif op[0] == "db_column":
                opstring += "db_column = {0}".format(op[1])
            else:
                sys.log("Failed to create filed option " + op + "!", withError = True)
                continue

            if not op[0] == ops[len(ops) - 1][0]:
                opstring += ", "

        return opstring



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os

from mirage.flow import Workflow
from mirage.command import log, command
from mirage.generate.model_template import create_model_class

class DjangoModelMakeFlow(Workflow):

    def main(self):

        datatypes = """

Data Type:         Rerating Fields:

 string              CharField
 auto                AutoField
 auto64              BigAutoField
 int64               BigIntegerField
 binary              BinaryField
 bool                BooleanField
 char                CharField
 date                DateField
 datetime            DateTimeField
 decimal             DecimalField
 duration            DurationField
 interval            DurationField
 email               EmailField
 file                FileField
 path                FilePathFields
 float               FloatField
 image               ImageField
 int                 IntegerField
 integer             IntegerField
 ip                  GenericIPAddressField
 nullbool            NullBooleanField
 pint                PositiveIntegerField
 slug                SlugField
 text                TextField
 time                TimeField
 url                 URLField
 uuid                UUIDField"

"""
        print(datatypes)



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import shutil

from mirage.flow import Workflow
from mirage.command import log, command
from mirage.miragefile import source
from mirage import proj
from mirage.template import readme_md
from mirage.template import gitignore
from mirage.template import package_json


class DjangoStartupWorkFlow(Workflow):
    
    def constructor(self):
        self._project_name = None


    def main(self):
        # Check 

        try:
            self._check_before()
        except:
            return

        # Input information
        log("Please type your new Django application information.")

        # Check namespace
        try:
            self._project_name = log("Project name", withInput = True)
            self._check_namesapce(self._project_name)
        except:
            log("Project \"{0}\" is already exists.".format(self._project_name), withError = True,
                    errorDetail = "Please remove duplication of Django project namespace.")
            return

        version      = log("App version", withInput = True, default = "0.0.1")
        author       = log("Author name", withInput = True)
        email        = log("Email",       withInput = True)
        git_url      = log("Git URL",     withInput = True)
        license_name = log("License",     withInput = True)
        description  = log("Description", withInput = True)
        copyrightor  = log("Copyrightor", withInput = True, default = author)



        
        self._create_new_django_app()


        with proj.InDir("./" + self._project_name):
            
            # Generate .gitignore
            log("Generating gitignore...")
            self._create_template_git_project()

            # Generate README.md
            log("Generating readme...")
            self._create_docs()

            # Generate Miragefile
            log("Generating Miragefile...")
            self._create_miragefile(version, author, email, git_url, license_name, description, copyrightor)

            # Add remote repo
            log("Adding remote repository...")
            command("git remote add origin " + git_url)

            # Make shell directory
            os.mkdir("shell")


        with proj.InDir("./shell"):

            # Generate package.json
            log("Generating package configuration...")
            self._create_package_json(version, description, git_url, author, email, license_name)

            # Install webpack
            log("Installing assets builder...")
            command("yarn add --dev webpack")


        # Completed
        log("Completed!")
    

    def _create_new_django_app(self):
        command("django-admin startproject " + self._project_name)


    def _create_miragefile(self, version, author, email, git_url, license_name, description, copyrightors):    
        with open("Miragefile", "w") as f:
            f.write(source.create(self._project_name, version, author, email, git_url, license_name, description, copyrightors))
     

    
    def _create_package_json(self, version, description, git_repository, author_name, email, license_name):
        with open("package.json", "w") as f:
            data = package_json.src(self._project_name, version, description, git_repository, author_name, email, license_name)
            f.write(data)


    def _create_template_git_project(self):
        ignorance = gitignore.src()

        with open(".gitignore", "w") as f:
            f.write(ignorance)

        command("git init")


    def _create_docs(self):
        with open("README.md", "a") as readme:
            readme.write(readme_md.src(self._project_name))


    def _check_before(self):
        
        try:
            import django
        except ImportError:
            log("Failed to import Django!", withError = True,
                                errorDetail = "You have to install Django before creating a new Django project.")
            raise ImportError


    def _check_namesapce(self, name):
        if os.path.exists(name):
            raise FileExistsError


class DjangoCMSStartupWorkFlow(Workflow):

    def constructor(self):
        try:
            self._project_name = self._option
        except:
            self._project_name = None


    def main(self):

        if self._project_name == None:
            log("Please type your new Django CMS application name.")
            self._project_name = log("Django CMS name", withInput = True)

        try:
            self._check(self._project_name)
        except:
            log("Project {0} is already exists.".format(self._project_name), withError = True)

        
        self._create_new_django_app(self._project_name)

        with proj.InDir("./" + self._project_name):
            self._create_template_git_project(self._project_name)
            self._create_docs(self._project_name)


    def _create_new_django_app(self, name):
        log("Creating Django CMS application...")
        log("Please wait for a moment.")
        command("djangocms " + name)


    def _create_template_git_project(self, name):
        ignorance = gitignore.src()

        with open(".gitignore", "w") as f:
            f.write(ignorance)

        command("git init")


    def _create_docs(self, name):
        with open("README.md", "a") as readme:
            readme.write(readme_md.src(name))


    def _check(self, name):
        if os.path.exists(name):
            raise FileExistsError



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import textwrap

def create_url(app):
    return textwrap.dedent(
'''
from django.urls import path

urlpatterns = [
    path(r'^url_letter/', """YOUR_VIEW_CLASS""".as_view(), name='INSERT NAME HERE'),
]


''').format(app=app).strip()



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys
import enum
import functools
from mirage.core import Void
from mirage import system

# Flow Classies
from mirage import workflows


class ArgumentsParser(object):

    def __init__(self):

        # Arguments
        self._cmd = None            # ex. **new**
        self._sub_action = None     # ex. new:**cms**
        self._option = None         # ex. g **app**
        self._option_detail = None  # ex. g app **--basic**
        self._values = None         # ex. g **app api mail user**
        self.__insert_arguments()   # <== Insert real value

        # Exec flow
        self._exec_flow = None

        # Arguments Array
        self._arguments = (
            self._cmd,
            self._sub_action,
            self._option,
            self._option_detail,
            self._values
        )

        # Is Assessmented
        self._assessmented = False



    def add_argument(self, shorten_cmd: str, long_cmd: str, option: str, execute: str) -> Void:

        if self._assessmented: return

        # Check command

        if not self._cmd == shorten_cmd and not self._cmd == long_cmd:
            return

        if not self._sub_action is None:
            return

        if not option is None:
            if self._option == option:
                self._exec_flow = execute
                self._assessmented = True
        else:
            self._exec_flow = execute
            self._assessmented = True

        return


    def add_argument_with_subaction(self, base_shorten_cmd: str,
                                    base_long_cmd: str, action: str, option: str, execute: str) -> Void:

        if self._assessmented: return

        if not self._cmd == base_shorten_cmd and not self._cmd == base_long_cmd:
            return

        if not self._sub_action == action:
            return

        if not option is None:
            if self._option == option:
                self._exec_flow = execute
                self._assessmented = True
        else:
            self._exec_flow = execute
            self._assessmented = True


        return


    def parse(self) -> Void:
        # If there are no command, show usage.
        if len(sys.argv) == 1:
            instance = getattr(workflows, "UsageShow")(self._arguments)
            instance.run()
            return

        # Check excute function is not empty.
        if not self._exec_flow == None:
            instance = getattr(workflows, self._exec_flow)(self._arguments)
            instance.run()
            return

        else:
            system.log("Unable to invoke action \"{0}\"!".format(sys.argv[1]), withError = True)
            instance = getattr(workflows, "UsageShow")(self._arguments)
            instance.run()


    def __insert_arguments(self):
        # Get main command **new**:cms
        try: self._cmd = self.__colon_separate_cmd(sys.argv[1])
        except: pass

        # Get subaction new:**cms**
        try: self._sub_action = self.__colon_separate_action(sys.argv[1])
        except: pass

        # Get option and detail option
        # optin =           mi g **app**
        # detail option =   mi g app **--basic**
        try:
            self._option = sys.argv[2]

            if "--" in sys.argv[3]:
                self._option_detail = sys.argv[3]
        except: pass

        # Get values
        try:
            if "--" in sys.argv[3]:
                self._values = sys.argv[4:]
            else:
                self._values = sys.argv[3:]
        except: pass


    def __colon_separate_cmd(self, cmd_colon_value: str) -> str:
        if ":" in cmd_colon_value:
            return cmd_colon_value.split(":")[0]
        else:
            return cmd_colon_value


    def __colon_separate_action(self, cmd_colon_value: str) -> str:
        if ":" in cmd_colon_value:
            return cmd_colon_value.split(":")[1]
        else:
            raise ValueError



class DetailOptionParser():

    def __init__(self, detail_option):
        self._option_detail = detail_option
        self._excute = None

    def add_argument(self, option, excute):
        if self._option_detail == option:
            self._excute = excute

    def parse(self):
        self._excute.excute()



def reserve_as_command(*reserved_args):

    def _decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            for reserve in reserved_args:
                if args[1] == reserve:
                    re = func(*args, **kwargs)
                    return re
        return wrapper
    return _decorator



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import json
import warnings
from mirage         import fileable
from mirage.command import log
from mirage.system import warning


# New
import sys

def in_project():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    try:
        set_import_root()

        import manage

        return True
    except ImportError:
        return False
    except:
        return False


def in_app():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    try:
        set_import_root()

        import apps
        
        if os.path.isfile("apps.py"):
            return True
        else:
            return False
    except ImportError:
        return False
    except:
        return False


def set_import_root():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    sys.path.append("./")



def get_project_name():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    current_dir = os.getcwd()
    directories = os.listdir(".")
    app_name = "FAILED TO GET"
    
    if in_project():

        for directory in directories:
            try:
                os.chdir(directory)

                if os.path.isfile("settings.py"):
                    app_name = str(os.getcwd()).split("/")[-1]

                os.chdir(current_dir)
            except:
                pass
    else:
        app_name = "Out of project dir"
    
    return app_name



def get_app_list():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    
    apps = []
    list_dir = os.listdir(os.getcwd())
    current = os.getcwd()

    if not in_project():
        return

    for dir_file in list_dir:
        if os.path.isdir(dir_file):
            os.chdir(dir_file)
            if in_app():
                apps.append(dir_file)
            os.chdir(current)
        else:
            continue

    return apps


# Old
def project_name():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    
    status = False

    if os.path.isfile("manage.py"):
        status = True
    else:
        status = False

    if status:

        contents = os.listdir(os.getcwd())

        for test in contents:
            if os.path.isdir(test):
                current = os.getcwd()
                os.chdir(test)
                if os.path.isfile("settings.py"):
                    return str(test)
                os.chdir(current)


# Config

def load_djfile():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    
    with open("Miragefile", "r") as djfile:
        try:
            return json.load(djfile)
        except:
            raise Exception


def load_additional_conf():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)
    
    with open("Miragefile.addon", "r") as djfile:
        try:
            return json.load(djfile)
        except:
            raise Exception


def load_secret_conf():
    warning.warn("mirage.project is now pending deprecation.", warning.PendingDeprecationWarning)

    with open("Miragefile.secret", "r") as djfile:
        try:
            return json.load(djfile)
        except:
            raise Exception



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import inspect
import warnings
import sys
import textwrap
import functools
from subprocess import check_output, DEVNULL, STDOUT
from mirage.system import warning


def log(string,
            withError = False, errorDetail = None,
            withInput = False, withConfirm = False, default = None):
    
    warning.warn("command.log will be deprecated on next release!", warning.PendingDeprecationWarning)


    if withError:

        print('\033[31mMirage: ' + str(string) + '\033[0m')

        if not errorDetail == None:
            separator_begin = "===== Error Detail =======================================================\n"
            separator_end   = "==========================================================================\n"
            print('\033[31m' + separator_begin + errorDetail + "\n" + separator_end + '\033[0m')


    elif withInput:
        string = str(input('\033[32m' + str(string) + ' >> \033[0m'))
        
        if string == "" and default != None:
            return default
        else:
            return string

    elif withConfirm:
        print('\033[31mMirage: ' + str(string) + '\033[0m')

        while True:
            answer = input('\033[32m' + "Please respond with yes or no [Y/N/y/n]" + ' >> \033[0m').lower()

            if answer in [ "y", "Y", "yes", "Yes", "YES", "Yeah"]:
                return True
            elif answer in [ "n", "N", "no", "No", "NO", "Nope"]:
                return False        
    else:
        print('\033[32mMirage: \033[0m' + str(string))



def raise_error_message(func):

    warning.warn("command.raise_error_message will be deprecated on next release!", warning.PendingDeprecationWarning)


    try:
        errored_func = func.__name__
    except:
        errored_func = "Failed to get func information!"

    try:
        errored_obj  = str(func)
    except:
        errored_obj  = "Failed to get errored object information!"

    try:
        func_sig     = inspect.signature(func)
    except:
        func_sig     = "Failed to get functino signature!"

    try:
        exec_info    = str(sys.exc_info())
    except:
        exec_info    = "Failed to get exec info!"


    return textwrap.dedent("""
Python Information:

Exceute func name : {func_name}
Exec Information  : {exec_info}
Object Info       : {obj_inf}
Signature         : {func_signature}
    """).format(
        func_name = errored_func,
        exec_info = exec_info,
        obj_inf   = errored_obj,
        func_signature = func_sig
    ).strip()



def command(command, withOutput = False):

    warning.warn("command.command will be deprecated on next release!", warning.PendingDeprecationWarning)

    separated_cmds = command.split(" ")

    if withOutput:
        try:
            check_output(separated_cmds, stderr=STDOUT)
        except:
            log("Failed to exec " + command + "!", withError = True)
            return

    else:
        try:
            check_output(separated_cmds, stderr=DEVNULL)
        except:
            log("Failed to exec " + command + "!", withError = True)
            return



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from mirage.console.django_console import DjangoConsoleWorkFlow, DjangoDBConsoleWorkFlow
class DjangoConsole(DjangoConsoleWorkFlow): pass
class DjangoDBConsole(DjangoDBConsoleWorkFlow): pass


from mirage.projectstartup.react_app_create import ReactStartupWorkFlow
class ReactStartup(ReactStartupWorkFlow): pass

from mirage.projectstartup.django_app_create import StartupWorkFlow
class Startup(StartupWorkFlow): pass

from mirage.help.usage_flow import UsageShowWorkFlow, VersionShowWorkFlow
class UsageShow(UsageShowWorkFlow): pass
class VersionShow(VersionShowWorkFlow): pass

from mirage.db.migrate import DjangoMigrateWorkFlow
class DjangoMigrate(DjangoMigrateWorkFlow): pass

from mirage.db.reset import DjangoDBResetWorkFlow
class DjangoDBReset(DjangoDBResetWorkFlow): pass

from mirage.generate.app import DjangoAppMakeWorkFlow
class DjangoAppMake(DjangoAppMakeWorkFlow): pass

from mirage.backup.backup import DjangoBackupAppWorkFlow
class DjangoBackupApp(DjangoBackupAppWorkFlow): pass

from mirage.configure.configure import ConfigureWorkFlow
class Configure(ConfigureWorkFlow): pass

from mirage.destroy.destroy import DjangoDestroyWorkFlow
class DjangoDestroy(DjangoDestroyWorkFlow): pass

from mirage.server.debug_server import DjangoDebugServerWorkFlow
class DjangoDebugServer(DjangoDebugServerWorkFlow): pass

from mirage.generate.model import DjangoModelMakeWorkflow
class DjangoModelMake(DjangoModelMakeWorkflow): pass

from mirage.heroku.configure import DjangoHerokuConfigureWorkFlow
class DjangoHerokuConfigure(DjangoHerokuConfigureWorkFlow): pass

from mirage.server.debug_server import DjangoLaunchBrowserWorkflow
class DjangoLaunchBrowser(DjangoLaunchBrowserWorkflow): pass

from mirage.manage.executor import DjangoManagePyWorkflow
class DjangoManagePy(DjangoManagePyWorkflow): pass

from mirage.db.merge import DjangoMergeMigrationWorkFlow
class DjangoMergeMigration(DjangoMergeMigrationWorkFlow): pass

from mirage.transfer.transfer import MirageTransferWorkflow
class MirageTransfer(MirageTransferWorkflow): pass


###########################################################
# ALL IMPORT                                              #
###########################################################

from mirage.projectstartup.minimum_app_create import *

from mirage.confscript import *

from mirage.generate import *

from mirage.checker import *

from mirage.projectstartup import *




# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys


def log(string,
            withError = False, errorDetail = None,
            withInput = False, withConfirm = False, default = None):


    if withError:

        print('\033[31mMirage: ' + str(string) + '\033[0m')

        if not errorDetail == None:
            separator_begin = "===== Error Detail =======================================================\n"
            separator_end   = "==========================================================================\n"
            print('\033[31m' + separator_begin + errorDetail + "\n" + separator_end + '\033[0m')


    elif withInput:
        string = str(input('\033[32m' + str(string) + ' >> \033[0m'))

        if string == "" and default != None:
            return default
        else:
            return string

    elif withConfirm:
        print('\033[31mMirage: ' + str(string) + '\033[0m')

        while True:
            answer = input('\033[32m' + "Please respond with yes or no [Y/N/y/n]" + ' >> \033[0m').lower()

            if answer in [ "y", "Y", "yes", "Yes", "YES", "Yeah"]:
                return True
            elif answer in [ "n", "N", "no", "No", "NO", "Nope"]:
                return False

    else:
        print('\033[32mMirage: \033[0m' + str(string))



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import tempfile, sys, traceback
from urllib import request
from mirage.proj import InDir
from mirage.flow import Workflow
from mirage.core import Void
from mirage.version import __version__ as ver
from mirage import system as mys



class UpdateCheckFlow(Workflow):

    def constructor(self) -> Void:
        self._url = "https://raw.githubusercontent.com/shotastage/mirage-django-lts/master/mirage/version.py"


    def main(self) -> Void:
        mys.log("Checking mirage update...")

        with tempfile.TemporaryDirectory() as td:
            with InDir(td):
                sys.path.append(td)
                request.urlretrieve(self._url, "mg_version_check.py")

                try:
                    import mg_version_check
                except ImportError:
                    mys.log("Failed to get version information!", withError=True,
                            errorDetail=mys.raise_error_message(self.main, traceback.format_exc()))
                    return

                if ver == mg_version_check.__version__:
                    if ver == "0.1.9":
                        mys.log("New mirage framework tools available!")
                        mys.log("See, https://github.com/shotastage/mirage-django for detail.")

                        mys.log("LTS Update available!", withError=True)
                        print("This version: {0}".format(ver))
                        print("Available version: {0}".format(mg_version_check.__version__))
                else:
                    mys.log("LTS Update available!", withError=True)
                    print("This version: {0}".format(ver))
                    print("Available version: {0}".format(mg_version_check.__version__))



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from abc import ABCMeta, abstractmethod
from mirage.core import Void
from mirage import system as mys


class Workflow():

    def __init__(self, parser):
        self.inherite       = parser
        self._action        = self.inherite[0]
        self._subaction     = self.inherite[1]
        self._option        = self.inherite[2]
        self._option_detail = self.inherite[3]
        self._values        = self.inherite[4]
        self.Stepflows      = []
        self.constructor()


    def constructor(self) -> Void:
        pass

    def get_first_arg(self) -> str:
        try:
            return self._values[0]
        except:
            raise ValueError

    def register(self, flow) -> Void:
        self.Stepflows.append(flow)

    def main(self) -> bool:
        # Main flow struct
        return True

    def run(self) -> Void:
        self.main()

        # Flow
        for flow in self.Stepflows:
            try:
                flow.run()
            except:
                raise Exception



class Stepflow():
    def __init__(self, parser):
        self.inherite       = parser
        self._action        = parser._cmd
        self._subaction     = parser._sub_action
        self._option        = parser._option
        self._option_detail = parser._option_detail
        self._values        = parser._values
        self._flows         = []


    def add(self, func) -> Void:
        self._flows.append(func)

    def run(self) -> Void:
        for flow in self._flows:
            try:
                flow()
            except:
                mys.log("Something error has occured!", withError = True, errorDetail = mys.raise_error_message(flow))
                raise Exception



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import shutil
import sys
import time
import distutils

from mirage import system as mys
from mirage import proj

from mirage              import fileable
from mirage.flow         import Workflow
from mirage.workspace import storage

class DjangoCreateSuperUserWorkflow(Workflow):

    def main(self):
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):
            os.system("python manage.py createsuperuser")


class DjangoManagePyWorkflow(Workflow):

    def constructor(self):
        self._cmd = self._option

    def main(self):
        
        if self._cmd == "superuser":
            self.createsuperuser()
        else:
            self.excute_manage_cmd()


    def createsuperuser(self):
        self._cmd = "createsuperuser"
        self.excute_manage_cmd()


    def excute_manage_cmd(self):
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):
            os.system("python manage.py " + self._cmd)



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import uuid
import shutil
import sqlite3
import contextlib
from mirage import fileable
from mirage import system as mys
from mirage.proj import MirageEnvironment, MirageEnvironmentLevel


class MirageWorkspace():

    @staticmethod
    def initialize():

        with MirageEnvironment(MirageEnvironmentLevel.inproject):
            if not fileable.exists(".mirage"):
                mys.log("Creating Mirage workspace...")
                fileable.mkdir(".mirage")
                fileable.mkdir(".mirage/cache/")
                fileable.mkdir(".mirage/persistence/")


    @staticmethod
    def persist(file_path, domain):
        object_id = str(uuid.uuid4())
        absolute_path = os.getcwd() + "/" + file_path

        # Logger instance
        logger = mys.progress.Progress()

        with MirageEnvironment(MirageEnvironmentLevel.inproject):

            logger.write("Safety caching...", withLazy = True)
            fileable.cd(".mirage/cache/")
            fileable.copy(absolute_path, os.getcwd() + "/" + file_path, force = True)

            logger.update("Archiving files...", withLazy = True)
            shutil.make_archive(object_id, "zip", root_dir = absolute_path)
            
            fileable.move(object_id + ".zip", "../persistence/")

            MirageWorkspace.__write_persistence_file_meta(object_id, ".mirage/persistence/", domain)

            logger.update("Saving file completed!", withLazy = True)


    @staticmethod
    def cache(file):
        pass

    @staticmethod
    def __write_persistence_file_meta(object_id, path, domain):
        con = sqlite3.connect(MirageWorkspace.__db_connect())
        cur = con.cursor()

        MirageWorkspace.query_execute(con, cur,
            "CREATE TABLE IF NOT EXISTS mi_ws_persistences (uuid TEXT PRIMARY KEY, path TEXT, domain TEXT);")

        MirageWorkspace.query_execute(con, cur,
            "INSERT INTO mi_ws_persistences VALUES (\"{0}\", \"{1}\", \"{2}\");".format(object_id, path, domain))
        con.commit()
        con.close()


    @staticmethod
    def __db_connect():
        return MirageEnvironment.search_project_root() + "/.mirage/mirage_workspace.sqlite3"


    @staticmethod
    def query_execute(connection, cur, query):
        try:
            cur.execute(query)
        except sqlite3.Error as e:
            mys.log("SQLite3 error has occured!", withError = True, errorDetail = e.args[0])
        except:
            mys.log("Unknown error has occured!", withError = True,
                            errorDetail = mys.raise_error_message(MirageWorkspace.__write_persistence_file_meta))



# -*- coding: utf-8 -*-
"""
Copyright 2017-2020 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys

from mirage.djargparse import ArgumentsParser



def main():

    parser = ArgumentsParser()

    # Usage & Version
    parser.add_argument("h", "help", None, "UsageShow")
    parser.add_argument("v", "version", None, "VersionShow")

    # Commands
    parser.add_argument("new", "newproject", None, "Startup")
    parser.add_argument_with_subaction("new", "newproject", "react", None, "ReactStartup")
    parser.add_argument_with_subaction("new", "newproject", "ng", None, "NgStartupWorkFlow")
    parser.add_argument_with_subaction("new", "newproject", "mini", None, "MirageMinimumStartupWorkFlow")

    parser.add_argument("b", "backup", "app", "DjangoBackupApp")

    parser.add_argument("conf", "configure", None, "Configure")

    parser.add_argument("c", "console", None, "DjangoConsole")
    parser.add_argument_with_subaction("c", "console", "db", None, "DjangoDBConsole")

    parser.add_argument("d", "destroy", "app", "DjangoDestroy")

    parser.add_argument_with_subaction("db", "database", "migrate", None, "DjangoMigrate")
    parser.add_argument_with_subaction("db", "database", "reset", None, "DjangoDBReset")
    parser.add_argument_with_subaction("db", "database", "merge", None, "DjangoMergeMigration")

    # parser.add_argument("d", "destroy", None, "DjangoDestroy")

    parser.add_argument("g", "generate", "app", "DjangoAppMake")

    parser.add_argument("g", "generate", "model", "DjangoModelMake")

    parser.add_argument("g", "generate", "module", "ModuleCreateFlow")

    parser.add_argument("heroku", "heroku", "configure", "DjangoHerokuConfigure")

    parser.add_argument("m", "manage", None, "DjangoManagePy")

    parser.add_argument("s", "server", None, "DjangoDebugServer")
    parser.add_argument("browser", "internal-browser", None, "DjangoLaunchBrowser")

    parser.add_argument("t", "transfer", None, "MirageTransfer")

    parser.add_argument("+", "confscript", None, "MirageConfigScriptFlow")

    parser.add_argument("?", "inquiry", "update", "UpdateCheckFlow")
    parser.add_argument("?", "inquiry", "system", "SystemCheckFlow")

    # Excute
    parser.parse()



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from mirage import version

def usage_doc():
    return """
Mirage v{0}

Usage:
    mg [action] option <--sub-option> <inputs>

    mg [action]:[subaction] option <--sub-option> <inputs>


Commands:              Short:       Options:                      Descriptions:

    newproject         new                                        Create a new Django project.
    newproject:react   new:react                                  Create a new Django API project with React.js front-end.
    newproject:ng      new:ng                                     Create a new Django API project with Angular.
                                                --nebular         Create a new Angular project with Nebular.
                                                --material        Create a new Angular project with Material theme.
    backup             b            app         <app name>        Backup exsiting app.
    launch-browser     browser                  <URL>             Launch browser set as default by system.
    configure          conf                     <config type>     Generate miragefile or reconfig mirage.
    console            c                                          Launch Django Python shell.
    console:db         c:db                                       Launch databse shell.
    database:migrate   db:migrate                                 Make migrations and apply migrations.
    database:merge     db:merge                                   Discard & recreate migrations.
    database:reset     db:reset                                   Reset all database. ( Only debugging SQLite is supported. )
    generate           g            app         <app names...>    Create multiple Django apps at once.
    generate           g            model       <model class>     Create Django model class.
    generate           g            module      <module bane>     Create a new Python module with __init__.py
    heroku             heroku       configure                     Configure setting files for deploing to heroku.
    manage             m            test                          Run test of Django application.
    manage             m            superuser                     Create super user for Django admin.
    manage             m            <manage.py command>           Run manage.py command.
    server             s                                          Launch debugging server.
    file               f                                          Create a new Python source file with copyrights doc string.

    help               h                                          Show usage of Mirage.
    version            v                                          Print version information.
    inquiry            ?            update                        Check update.
    inquiry            ?            system                        Check platform and Python version.


""".format(version.__version__)


def version_doc():
    return """
Mirage Version {0}

https://github.com/shotastage/django-mirage

Copyright (c) 2017-2018 Shota Shimazu
This software is licensed under the Apache v2, see LICENSE for detail.
""".format(version.__version__)



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""


import os

from mirage.flow import Workflow
from mirage import project
from mirage.command import log
from mirage.command import command


class DjangoDBResetWorkFlow(Workflow):

    def main(self):
        log("Clearing all DB...")
        self._reset_db()

    def _reset_db(self):

        msg = "This action will remove ALL stored data in database and data can't be restored!\nDo you confirm to continue this action?"

        if project.in_project():
            if log(msg, withConfirm = True):
                self._remove_sqlite()
            else:
                log("Canceled.")
    
        else:
            log("Failed to reset database.", withError = True, errorDetail = """
Django Console Database Manager Error!s

Currently, Django Console support SQLite database for debug.
            """)
                

    def _remove_sqlite(self):
        log("Removing SQLite3 file...")
        if os.path.exists("db.sqlite3"):
            os.remove("db.sqlite3")



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import sys
import enum
import pathlib
from mirage.core import Void
from mirage import proj
from mirage import fileable
from mirage.system import warning, log
from mirage.miragefile import conf


class MirageEnvironment():

    def __init__(self, env_level):
        self._current = os.getcwd()
        self._level = env_level


    def __enter__(self):

        if self._level == MirageEnvironmentLevel.inproject:
            proj_root = MirageEnvironment.search_project_root()
            os.chdir(proj_root)
            return

        if self._level == MirageEnvironmentLevel.indjango:
            os.chdir(MirageEnvironment.search_project_root())
            django_root = conf.Config().get(conf.Category.django, conf.Detail.django_path)
            os.chdir(django_root)
            return

        if self._level == MirageEnvironmentLevel.inapp:
            app_root = MirageEnvironment.search_app_root()
            os.chdir(app_root)
            return


    def __exit__(self, exception_type, exception_value, traceback):
        os.chdir(self._current)



    @staticmethod
    def search_project_root():
        """
        Search your Django project root.

        returns:
            - path:string  Django project root path
        """

        while True:

            current = os.getcwd()

            if pathlib.Path("Miragefile.py").is_file() or pathlib.Path("Miragefile").is_file():
                return current
            elif os.getcwd() == "/":
                raise FileNotFoundError
            else:
                os.chdir("../")

    @staticmethod
    def search_app_root():
        """
        Search your Django application root

        returns:
            - (String) Django application root path
        """
        while True:

            current = os.getcwd()

            if pathlib.Path("apps.py").is_file():
                return current
            elif pathlib.Path.cwd() == "/":
                raise FileNotFoundError
            else:
                os.chdir("../")


    @staticmethod
    def set_import_root() -> Void:
        """
        Set path to import current dir Python module.
        """
        sys.path.append("./")


    @staticmethod
    def in_django_project() -> bool:
        """
        Judge where current working directory is in Django project or not.

        returns:
            - (Bool) cwd is in proj dir returns True
        """
        try:
            MirageEnvironment.set_import_root()
            import manage
            return True
        except ImportError:
            return False
        except:
            return False


    @staticmethod
    def in_project() -> bool:
        if pathlib.Path("Miragefile").is_file():
            return True
        elif pathlib.Path("Miragefile.py").is_file():
            return True
        else:
            return False


    @staticmethod
    def in_app() -> bool:
        """
        Judge where current working directory is in Django application or not.

        returns:
            - (Bool) cwd is in app dir returns True
        """
        try:
            MirageEnvironment.set_import_root()
            import apps
            if os.path.isfile("apps.py"):
                return True
            else:
                return False
        except ImportError:
            return False
        except:
            return False


    @staticmethod
    def get_app_list() -> list:
        """
        Return application lists.

        returns:
            - (Array<String>) cwd is in app dir returns True
        """

        warning.warn("This method is now under construction.", warning.UnderConstructionWarning)

        apps = []
        list_dir = os.listdir(os.getcwd())
        current = os.getcwd()

        if not MirageEnvironment.in_project(): return

        for dir_file in list_dir:
            if os.path.isdir(dir_file):
                os.chdir(dir_file)
                if MirageEnvironment.in_app():
                    apps.append(dir_file)
                os.chdir(current)
            else:
                continue

        return apps


class MirageEnvironmentLevel(enum.Enum):
    inproject   = 0
    indjango    = 1
    inapp       = 2
    outproject  = 3


class WorkingLevel(enum.Enum):
    inproject   = 0
    indjango    = 1
    inapp       = 2
    outproject  = 3



# -*- encoding:utf-8 -*-
from setuptools import setup, find_packages
from mirage.version import __version__ as ver
import sys


sys.path.append('./mirage')
sys.path.append('./tests')

if __name__ == "__main__":
    setup(
        name = "mirage-django-lts",
        version = ver,
        author = "Shota Shimazu",
        author_email = "hornet.live.mf@gmail.com",
        classifiers=[
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
        ],
        packages = find_packages(exclude=('tests', 'shell')),
        include_package_data = True,
        zip_safe = False,
        install_requires = [
            # "Flask"
        ],
        entry_points = {
            'console_scripts':[
                'mirage = mirage.mirage:main',
                'mg = mirage.mirage:main',
                'mgl = mirage.mirage:main',
            ],
        },
        description = "Advanced Django Console (LTS Version)",
        long_description = "Advanced extended command line tool for Django.",
        url = "https://github.com/shotastage/mirage-django-lts/",
        license = "Apache",
        platforms = ["POSIX", "Windows", "Mac OS X"],
        test_suite = "mirage_test.suite",
    )



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import shutil
import datetime
from functools import lru_cache
from mirage import proj
from mirage import fileable
from mirage.flow import Workflow
from mirage import system as mys
from mirage.template import readme_md, gitignore, package_json
from mirage.miragefile import source, source2, source_secret



class NgStartupWorkFlow(Workflow):

    def constructor(self):
        self._plugins = self._option


    def main(self):

        # Check
        try:
            self._check_before()
        except:
            return

        # Input information
        mys.log("Please type your new Django application information.")

        # Check namespace
        try:
            self._project_name = mys.log("Project name", withInput = True)
            self._check_namesapce(self._project_name)
        except:
            mys.log("Project \"{0}\" is already exists.".format(self._project_name), withError = True,
                    errorDetail = "Please remove duplication of Django project namespace.")
            return

        version      = mys.log("App version", withInput = True, default = "0.0.1")
        author       = mys.log("Author name", withInput = True)
        email        = mys.log("Email",       withInput = True)
        git_url      = mys.log("Git URL",     withInput = True)
        license_name = mys.log("License",     withInput = True)
        license_url  = mys.log("License Url", withInput = True)
        description  = mys.log("Description", withInput = True)
        copyrightor  = mys.log("Copyrightor", withInput = True, default = author)




        self._create_new_django_app()

        # Create logging instance
        logger = mys.Progress()

        with proj.InDir("./" + self._project_name):

            # Generate .gitignore
            #log("Generating gitignore...")
            logger.write("Generating gitignore...", withLazy = True)
            self._create_template_git_project()

            # Generate README.md
            logger.update("Generating readme...", withLazy = True)
            self._create_docs(author, description, license_name)

            # Generate Miragefile
            logger.update("Generating Miragefile...", withLazy = True)
            self._create_miragefile(version, author, email, git_url,
                                    license_name, license_url, description, copyrightor)

            # Add remote repo
            logger.update("Adding remote repository...", withLazy = True)
            mys.command("git remote add origin " + git_url)

            # Create React App
            self._create_package_json()
            logger.update("Preparing Angular setup tools...", withLazy=True)
            logger.update("Preparing Angular setup tools... ( please wait for a momenet )")
            mys.command("yarn add --dev @angular/cli")

            # To avoid ng inside the project error, remove package.package.json
            os.remove("package.json")

            logger.update("Configuring Angular setup tools to use Yarn package...")
            mys.command("./node_modules/.bin/ng set --global packageManager=yarn")
            logger.update("Creating Angular app...", withLazy = True)
            logger.update("Creating Angular app... ( please wait for a momenet )")
            mys.command("./node_modules/.bin/ng new shell")


            if self._plugins == "--nebular":
                logger.update("Installing Nebular packages...", withLazy = True)
                logger.update("Installing Nebular packages... ( please wait for a momenet )")
                with proj.InDir("./shell"):
                    mys.command("yarn add @nebular/theme @nebular/auth")

            elif self._plugins == "--material":
                logger.update("Installing material theme packages...")
                with proj.InDir("./shell"):
                    mys.command("yarn add @angular/material @angular/cdk")

            elif self._plugins == "--bootstrap":
                logger.update("Installing Bootstrap theme packages...")
                with proj.InDir("./shell"):
                    mys.command("yarn add ngx-bootstrap")


            # Cleaning
            logger.update("Cleaning...", withLazy = True)
            fileable.rm("yarn.lock")
            fileable.rm("package.json")
            fileable.rm("node_modules/")

            with proj.InDir("./shell"):
                fileable.rm(".gitignore")
                fileable.rm("README.md")


        # Completed
        logger.update("Completed!")


    def _create_new_django_app(self):
        mys.command("django-admin startproject " + self._project_name)


    def _create_miragefile(self, version, author, email, git_url, license_name, license_url, description, copyrightors):
        with open("Miragefile", "w") as f:
            f.write(source.create(self._project_name, version, author, email, git_url, license_name, description, copyrightors))

        with open("Miragefile@next.py", "w") as f:
            f.write(source2.create(self._project_name, version, author, email, git_url, license_name, description, copyrightors))

        with open("Miragefile.secret", "w") as f:
            f.write(source_secret.create(author, email, license_url))


    def _create_package_json(self):
        with open("package.json", "w") as f:
            f.write('{"name": "tmpapp", "version": "0.0.1"}')


    def _create_template_git_project(self):
        ignorance = gitignore.src()

        with open(".gitignore", "w") as f:
            f.write(ignorance)

        mys.command("git init")


    def _create_docs(self, author, description, license_name):

        with open("README.md", "a") as readme:
            readme.write(readme_md.src(self._project_name, description))


    @lru_cache(maxsize = 10)
    def _get_current(self) -> str:
        return datetime.datetime.now().strftime("%Y/%m/%d")


    def _check_before(self):

        try:
            import django
        except ImportError:
            mys.log("Failed to import Django!", withError = True,
                    errorDetail = "You have to install Django before creating a new Django project.")
            raise ImportError


    def _check_namesapce(self, name):
        if os.path.exists(name):
            raise FileExistsError



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import shutil
import zipfile
import hashlib
from pathlib import Path
from mirage.flow import Workflow
from mirage import system as mys
from mirage import proj


class MirageTransferWorkflow(Workflow):
    
    def constructor(self):
        self._app = self._option
        try:
            self._to  = self._values[0]
        except:
            self._to = mys.log("Trasfer to", withInput = True)

    def main(self):
        # Beta Warning
        mys.log("This feature is now under Beta version.", withError=True)

        # Logger instance

        logger = mys.progress.Progress()
        
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):

            # In advance checking.
            try:
                self._check(logger)
            except:
                return

            logger.write("Searching application %s..." % self._app, withLazy = True)

            if Path(self._app).is_dir():
                mys.log(os.getcwd())
                shutil.copytree(self._app, os.path.join(self._to, self._app))
                logger.update("Completed!")
            else:
                logger.update("Can not detect application %s!" % self._app)
                mys.log("Failed to transfer app %s!" % self._app, withError = True,
                errorDetail = mys.raise_error_message(
                    "The application diresctory does not exists!"
                ))
                return

    def _check(self, logger):
        
        with proj.InDir(self._to):

            if Path(self._app).is_dir():
                if not mys.log("%s ia already exists.\nAre you sure to replace new one?" % self._app, withConfirm = True):
                    raise FileExistsError
                else:
                    os.system("rm -rf " + self._app)


    def _compress(self, logger):
        logger.update("Compressing application...")

        try:
            shutil.make_archive(self._app, "zip", root_dir = os.getcwd())
        except:
            pass
            # shutil.make_archive raises ZIP does not support timestamps before 1980
            #

    
    def _move(self, logger):
        logger.update("Transfering application...")
        shutil.move(self._app, self._to)

    def _extract(self, logger):

        with proj.InDir(self._to):
            logger.update("Extracting application...")

            try:
                with zipfile.ZipFile(self._app + ".zip", "r") as contents:
                    contents.extractall()
            except:
                pass
                # zipfile.ZipFile raises No such file or directory: 'hoge.zip'
                # But, zip file is extracted!

            logger.update("Cleaning...")
            #os.remove(self._app + ".zip")


    def _clean(self, logger):
        pass
        # mys.log(os.getcwd(), withError=True)
        # logger.update("Cleaning...")
        # os.remove(self._app + ".zip")



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import textwrap
import datetime


def create(proj_name, proj_ver, proj_author, author_email, git_url,
                                        proj_license, proj_desc, copyrightor):

    return textwrap.dedent('''
#
# Mirage Configuration Scripts Version 2018/03/23
# Copyright (c) 2017-2018 Shota Shimazu
# This software is licensed under the Apache v2, see LICENSE for detail.
#

from mirage import system as mys
from mirage.core import Void
from mirage.miragefile.conf import Category, Detail, Config
from mirage.confscript import ConfigScript
from mirage.confscript.settings import Settings


MIRAGE_CONFIG_SCRIPT_VERSION = "0.0.1"
MIRAGE_CONFIG_DEFAULT_CLASS = "MirageConfig"


class MirageConfig(ConfigScript):

    BASIC_PROJECT = {{
        "NAME": "{PROJECT_NAME}",
        "VERSION": "{PROJECT_VERSION}",
        "AUTHOR": "{PROJECT_AUTHOR} <{PROJECT_AUHOR_EMAIL}>",
        "GIT_URL": "{GIT_URL}",
        "LICENSE": "{PROJECT_LICENSE}",
        "DESCRIPTION": "{PROJECT_DESC}",
    }}

    DJANGO_PROJECT = {{
        "path": ".",
        "module": "{PROJECT_NAME}",
        "package": "pipenv",
        "database": "PostgreSQL",
    }}

    FRONT_END_PROJECT = {{
        "path": "shell",
        "package": "yarn",
        "builder": "webpack",
    }}

    COPYRIGHT = {{
        "start_year": {COPYRIGHT_START},
        "copyrightors": [
            "{COPYRIGHTOR}",
        ]
    }}



    def initialize(self) -> Void:
        mys.log("Mirage Setting Script v0.0.1")


    def main(self) -> int:
        # Add custom CLI script written in Python3
        # self.register_custom_command("custom-command", "<option>", "tools/scripts/custom_script.py")

        # Add custom CLI script written in other programming language.
        # self.register_custom_command_with_runtime("custom-command", "tools/scripts/custom_script.rb", "ruby")

        return 0


    def deinitialize(self) -> Void:
        mys.log("Bye : )")

''').format(
    PROJECT_NAME        = proj_name,
    PROJECT_VERSION     = proj_ver,
    PROJECT_AUTHOR      = proj_author,
    PROJECT_AUHOR_EMAIL = author_email,
    GIT_URL             = git_url,
    PROJECT_LICENSE     = proj_license,
    PROJECT_DESC        = proj_desc,
    MODULE_NAME         = proj_name,
    COPYRIGHT_START     = get_start_year(),
    COPYRIGHTOR         = copyrightor,
).strip()


def get_start_year():
    return datetime.datetime.now().strftime("%Y")



import os
import sys

from mirage import proj
from mirage import system as mys
from mirage.flow import Workflow


class DjangoConsoleWorkFlow(Workflow):

    def main(self):
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):
            mys.log("Launching Django Python Shell")
            os.system("python manage.py shell")


class DjangoDBConsoleWorkFlow(Workflow):

    def main(self):
        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.indjango):
            mys.log("Launching Django Python Shell")
            os.system("python manage.py dbshell")



# -*- coding: utf-8 -*-
"""
Copyright 2017-2018 Shota Shimazu.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import enum
import json
from mirage import proj
from mirage import system as mys
from mirage.exceptions import MiragefileUnknownError
from functools import lru_cache


class Category(enum.Enum):
    project_basic   = 0
    django          = 1
    frontend        = 2
    workspace       = 3
    copyright       = 4
    private_profile = 5


class Detail(enum.Enum):
    project_name    = 0
    project_version = 1
    project_author  = 2
    project_git     = 3
    project_license = 4
    project_description = 5
    django_path     = 6
    django_module   = 7
    django_package_manager = 8
    django_db_backend = 9
    copyright_start_year = 10
    copyright_copyrigtors = 11
    private_name = 12
    private_license_url = 13


class Config():

    def __init__(self, file_type = None):

        with proj.MirageEnvironment(proj.MirageEnvironmentLevel.inproject):
            if file_type == "secret":
                self._data = self._load_json("Miragefile.secret")
            elif file_type == "addon":
                self._data = self._load_json("Miragefile.addon")
            elif file_type == None:
                self._data = self._load_json("Miragefile")
            else:
                mys.log("Wrong configuration type {0}.".format(file_type), withError = True)


    @lru_cache(maxsize = 100)
    def get(self, category, detail):
        
        try:
            if category == Category.project_basic:
                return self._get_project(detail)
            if category == Category.django:
                return self._get_django(detail)
            if category == Category.copyright:
                return self._get_copyright(detail)
            if category == Category.private_profile:
                return self._get_private_profile(detail)
        except:
            mys.log("Failed to get value from Miragefile!", withError = True)


    @lru_cache(maxsize = 100)
    def _get_project(self, detail):
       
        if detail == Detail.project_name:
            return self._data["project"]["name"]
        elif detail == Detail.project_version:
            return self._data["project"]["version"]
        elif detail == Detail.project_author:
            return self._data["project"]["author"]
        elif detail == Detail.project_git:
            return self._data["project"]["git"]
        elif detail == Detail.project_license:
            return self._data["project"]["license"]
        elif detail == Detail.project_description:
            return self._data["project"]["description"]
        else:
            mys.log("The config information named " + str(detail) + " does not exist!", withError = True) 
            raise MiragefileUnknownError


    @lru_cache(maxsize = 100)
    def _get_django(self, detail):

        if detail == Detail.django_path:
            return self._data["project"]["django"]["path"]
        elif detail == Detail.django_module:
            return self._data["project"]["django"]["module"]
        elif detail == Detail.django_package_manager:
            return self._data["project"]["django"]["package"]
        elif detail == Detail.django_db_backend:
            return self._data["project"]["django"]["database"]
        else:
            mys.log("The config information named " + str(detail) + " does not exist!", withError = True) 
            raise MiragefileUnknownError


    @lru_cache(maxsize = 100)
    def _get_copyright(self, detail):

        if detail == Detail.copyright_start_year:
            return self._data["project"]["copyright"]["start_year"]
        elif detail == Detail.copyright_copyrigtors:
            return self._data["project"]["copyright"]["copyrightors"]
        else:
            mys.log("The config information named " + detail + " does not exist!", withError = True) 
            raise MiragefileUnknownError


    @lru_cache(maxsize = 100)
    def _get_private_profile(self, detail):

        if detail == Detail.private_name:
            return self._data["private_profile"]["name"]
        elif detail == Detail.private_license_url:
            return self._data["private_license"]["url"]
        else:
            mys.log("The config information named " + detail + " does not exist!", withError = True) 
            raise MiragefileUnknownError


    @lru_cache(maxsize = 100)
    def _load_json(self, filename):
        if not os.path.exists(filename):
            mys.log("Failed to find Miragefile!", withError = True)
            raise FileNotFoundError

        with open(filename, "r") as jsonfile:
            try:
                return json.load(jsonfile)
            except:
                mys.log("Failed to load Miragefile!", withError = True, errorDetail = mys.raise_error_message(self._load_json))
                raise FileNotFoundError
